<?php

declare(strict_types=1);

namespace Dedi\SyliusQuotePlugin\EventListener\Sales;

use Dedi\SyliusQuotePlugin\Entity\OrderInterface;
use Dedi\SyliusQuotePlugin\Entity\OrderItemInterface;
use Dedi\SyliusQuotePlugin\Service\QuoteConfigurationAwareInterface;
use Dedi\SyliusQuotePlugin\Service\QuoteConfigurationInterface;
use Dedi\SyliusQuotePlugin\Specification\Order\IsFromQuoteSpecification as OrderIsFromQuoteSpecification;
use Dedi\SyliusQuotePlugin\Specification\OrderItem\IsFromQuoteSpecification as OrderItemIsFromQuoteSpecification;
use InvalidArgumentException;
use Sylius\Bundle\ResourceBundle\Event\ResourceControllerEvent;
use Sylius\Component\Order\Context\CartContextInterface;

class OrderItemQuoteLockListener implements QuoteConfigurationAwareInterface
{
    private QuoteConfigurationInterface $quoteConfiguration;

    private CartContextInterface $cartContext;

    private OrderIsFromQuoteSpecification $orderIsFromQuoteSpecification;

    private OrderItemIsFromQuoteSpecification $orderItemIsFromQuoteSpecification;

    public function __construct(
        QuoteConfigurationInterface $quoteConfiguration,
        CartContextInterface $cartContext,
        OrderIsFromQuoteSpecification $orderIsFromQuoteSpecification,
        OrderItemIsFromQuoteSpecification $orderItemIsFromQuoteSpecification,
    ) {
        $this->quoteConfiguration = $quoteConfiguration;
        $this->orderItemIsFromQuoteSpecification = $orderItemIsFromQuoteSpecification;
        $this->cartContext = $cartContext;
        $this->orderIsFromQuoteSpecification = $orderIsFromQuoteSpecification;
    }

    public function __invoke(ResourceControllerEvent $event)
    {
        if ($this->quoteConfiguration->isCartFromQuoteEditAllowed()) {
            return;
        }

        $orderItem = $event->getSubject();

        if (!$orderItem instanceof OrderItemInterface) {
            throw new InvalidArgumentException(sprintf(
                'Expected an instance of "%s", got "%s"',
                OrderItemInterface::class,
                get_class($orderItem),
            ));
        }

        /** @var ?OrderInterface $order */
        $order = $orderItem->getOrder();

        if (null !== $order && $this->orderItemIsFromQuoteSpecification->isSatisfiedBy($orderItem)) {
            $event->stop('dedi_quote.cart.cannot_update_from_quote');
        }

        if (null === $order) {
            $cart = $this->cartContext->getCart();
            if ($this->orderIsFromQuoteSpecification->isSatisfiedBy($cart)) {
                $event->stop('dedi_quote.cart.cannot_update_from_quote');
            }
        }
    }

    public function getQuoteConfiguration(): QuoteConfigurationInterface
    {
        return $this->quoteConfiguration;
    }
}
